@@ -34,14 +34,18 @@ The ray is expressed in coordinates local to the volume (i.e. texture
 coordinates).
 
 """
+from __future__ import annotations
+
+from typing import Optional
 from functools import lru_cache
 
-from ._scalable_textures import CPUScaledTexture3D, GPUScaledTextured3D
+from ._scalable_textures import CPUScaledTexture3D, GPUScaledTextured3D, Texture2D
 from ..gloo import VertexBuffer, IndexBuffer
 from ..gloo.texture import should_cast_to_f32
 from . import Visual
 from .shaders import Function
 from ..color import get_colormap
+from ..io import load_spatial_filters
 
 import numpy as np
 
@@ -141,16 +145,16 @@ vec4 calculateColor(vec4 betterColor, vec3 loc, vec3 step)
 
     // calculate normal vector from gradient
     vec3 N; // normal
-    color1 = $sample( u_volumetex, loc+vec3(-step[0],0.0,0.0) );
-    color2 = $sample( u_volumetex, loc+vec3(step[0],0.0,0.0) );
+    color1 = $get_data(loc+vec3(-step[0],0.0,0.0) );
+    color2 = $get_data(loc+vec3(step[0],0.0,0.0) );
     N[0] = colorToVal(color1) - colorToVal(color2);
     betterColor = max(max(color1, color2),betterColor);
-    color1 = $sample( u_volumetex, loc+vec3(0.0,-step[1],0.0) );
-    color2 = $sample( u_volumetex, loc+vec3(0.0,step[1],0.0) );
+    color1 = $get_data(loc+vec3(0.0,-step[1],0.0) );
+    color2 = $get_data(loc+vec3(0.0,step[1],0.0) );
     N[1] = colorToVal(color1) - colorToVal(color2);
     betterColor = max(max(color1, color2),betterColor);
-    color1 = $sample( u_volumetex, loc+vec3(0.0,0.0,-step[2]) );
-    color2 = $sample( u_volumetex, loc+vec3(0.0,0.0,step[2]) );
+    color1 = $get_data(loc+vec3(0.0,0.0,-step[2]) );
+    color2 = $get_data(loc+vec3(0.0,0.0,step[2]) );
     N[2] = colorToVal(color1) - colorToVal(color2);
     betterColor = max(max(color1, color2),betterColor);
     float gm = length(N); // gradient magnitude
@@ -267,7 +271,7 @@ void main() {
             if (distance_from_clip >= 0)
             {
                 // Get sample color
-                vec4 color = $sample(u_volumetex, loc);
+                vec4 color = $get_data(loc);
                 float val = color.r;
                 texture_sampled = true;
 
@@ -382,7 +386,7 @@ _MIP_SNIPPETS = dict(
 
             vec3 small_step = step * 0.1;
             for (int i=0; i<10; i++) {
-                float val = $sample(u_volumetex, loc).r;
+                float val = $get_data(loc).r;
                 if ( val > maxval) {
                     maxval = val;
                     max_loc_tex = start_loc_refine + (small_step * i);
@@ -449,7 +453,7 @@ _MINIP_SNIPPETS = dict(
 
             vec3 small_step = step * 0.1;
             for (int i=0; i<10; i++) {
-                float val = $sample(u_volumetex, loc).r;
+                float val = $get_data(loc).r;
                 if ( val < minval) {
                     minval = val;
                     min_loc_tex = start_loc_refine + (small_step * i);
@@ -520,7 +524,7 @@ _ISO_SNIPPETS = dict(
             // Take the last interval in smaller steps
             vec3 iloc = loc - step;
             for (int i=0; i<10; i++) {
-                color = $sample(u_volumetex, iloc);
+                color = $get_data(iloc);
                 if (color.r > u_threshold) {
                     color = calculateColor(color, iloc, dstep);
                     gl_FragColor = applyColormap(color.r);
@@ -561,6 +565,19 @@ _AVG_SNIPPETS = dict(
         """,
 )
 
+_INTERPOLATION_TEMPLATE = """
+    #include "misc/spatial-filters.frag"
+    vec4 texture_lookup_filtered(vec3 texcoord) {
+        // no need to discard out of bounds, already checked during raycasting
+        return %s($texture, $shape, texcoord);
+    }"""
+
+_TEXTURE_LOOKUP = """
+    vec4 texture_lookup(vec3 texcoord) {
+        // no need to discard out of bounds, already checked during raycasting
+        return texture3D($texture, texcoord);
+    }"""
+
 
 class VolumeVisual(Visual):
     """Displays a 3D Volume
@@ -593,8 +610,16 @@ class VolumeVisual(Visual):
     gamma : float
         Gamma to use during colormap lookup.  Final color will be cmap(val**gamma).
         by default: 1.
-    interpolation : {'linear', 'nearest'}
-        Selects method of image interpolation.
+    interpolation : str
+        Selects method of texture interpolation. Makes use of the two hardware
+        interpolation methods and the available interpolation methods defined
+        in vispy/gloo/glsl/misc/spatial_filters.frag
+
+            * 'nearest': Default, uses 'nearest' with Texture interpolation.
+            * 'linear': uses 'linear' with Texture interpolation.
+            * 'hanning', 'hamming', 'hermite', 'kaiser', 'quadric', 'bicubic',
+                'catrom', 'mitchell', 'spline16', 'spline36', 'gaussian',
+                'bessel', 'sinc', 'lanczos', 'blackman'
     texture_format : numpy.dtype | str | None
         How to store data on the GPU. OpenGL allows for many different storage
         formats and schemes for the low-level texture data stored in the GPU.
@@ -641,8 +666,6 @@ class VolumeVisual(Visual):
 
     """
 
-    _interpolation_methods = ['linear', 'nearest']
-
     _rendering_methods = {
         'mip': _MIP_SNIPPETS,
         'minip': _MINIP_SNIPPETS,
@@ -663,6 +686,11 @@ class VolumeVisual(Visual):
         'fragment': _FRAGMENT_SHADER,
     }
 
+    _func_templates = {
+        'texture_lookup_interpolated': _INTERPOLATION_TEMPLATE,
+        'texture_lookup': _TEXTURE_LOOKUP,
+    }
+
     def __init__(self, vol, clim="auto", method='mip', threshold=None,
                  attenuation=1.0, relative_step_size=0.8, cmap='grays',
                  gamma=1.0, interpolation='linear', texture_format=None,
@@ -688,7 +716,19 @@ class VolumeVisual(Visual):
         # Create gloo objects
         self._vertices = VertexBuffer()
 
+        kernel, interpolation_methods = load_spatial_filters()
+        self._kerneltex = Texture2D(kernel, interpolation='nearest')
+        interpolation_methods, interpolation_fun = self._init_interpolation(
+            interpolation_methods)
+        self._interpolation_methods = interpolation_methods
+        self._interpolation_fun = interpolation_fun
         self._interpolation = interpolation
+        if self._interpolation not in self._interpolation_methods:
+            raise ValueError("interpolation must be one of %s" %
+                             ', '.join(self._interpolation_methods))
+        self._data_lookup_fn = None
+        self._need_interpolation_update = True
+
         self._texture = self._create_texture(texture_format, vol)
         # used to store current data for later CPU-side scaling if
         # texture_format is None
@@ -734,17 +774,39 @@ class VolumeVisual(Visual):
 
         self.freeze()
 
+    def _init_interpolation(self, interpolation_methods):
+        # create interpolation shader functions for available
+        # interpolations
+        fun = [Function(self._func_templates['texture_lookup_interpolated'] % (n + '3D'))
+               for n in interpolation_methods]
+        interpolation_methods = [n.lower() for n in interpolation_methods]
+
+        interpolation_fun = dict(zip(interpolation_methods, fun))
+        interpolation_methods = tuple(sorted(interpolation_methods))
+
+        # overwrite "nearest" and "linear" spatial-filters
+        # with  "hardware" interpolation _data_lookup_fn
+        hardware_lookup = Function(self._func_templates['texture_lookup'])
+        interpolation_fun['nearest'] = hardware_lookup
+        interpolation_fun['linear'] = hardware_lookup
+        return interpolation_methods, interpolation_fun
+
     def _create_texture(self, texture_format, data):
         if texture_format is not None:
             tex_cls = GPUScaledTextured3D
         else:
             tex_cls = CPUScaledTexture3D
 
+        if self._interpolation == 'linear':
+            texture_interpolation = 'linear'
+        else:
+            texture_interpolation = 'nearest'
+
         # clamp_to_edge means any texture coordinates outside of 0-1 should be
         # clamped to 0 and 1.
         # NOTE: This doesn't actually set the data in the texture. Only
         # creates a placeholder texture that will be resized later on.
-        return tex_cls(data, interpolation=self._interpolation,
+        return tex_cls(data, interpolation=texture_interpolation,
                        internalformat=texture_format,
                        format='luminance',
                        wrapping='clamp_to_edge')
@@ -796,10 +858,6 @@ class VolumeVisual(Visual):
             self._need_vertex_update = True
         self._vol_shape = shape
 
-    @property
-    def interpolation_methods(self):
-        return self._interpolation_methods
-
     @property
     def rendering_methods(self):
         return list(self._rendering_methods)
@@ -856,57 +914,80 @@ class VolumeVisual(Visual):
         self.shared_program['texture2D_LUT'] = self.cmap.texture_lut()
         self.update()
 
+    @property
+    def interpolation_methods(self):
+        return self._interpolation_methods
+
     @property
     def interpolation(self):
-        """The interpolation method to use
-
-        Current options are:
-
-            * linear: this method is appropriate for most volumes as it creates
-              nice looking visuals.
-            * nearest: this method is appropriate for volumes with discrete
-              data where additional interpolation does not make sense.
-        """
+        """Get interpolation algorithm name."""
         return self._interpolation
 
     @interpolation.setter
-    def interpolation(self, interp):
-        if interp not in self._interpolation_methods:
-            raise ValueError(
-                "interpolation must be one of %s"
-                % ', '.join(self._interpolation_methods)
-            )
-        if self._interpolation != interp:
-            self._interpolation = interp
-            self._texture.interpolation = self._interpolation
+    def interpolation(self, i):
+        if i not in self._interpolation_methods:
+            raise ValueError("interpolation must be one of %s" %
+                             ', '.join(self._interpolation_methods))
+        if self._interpolation != i:
+            self._interpolation = i
+            self._need_interpolation_update = True
             self.update()
 
+    # The interpolation code could be transferred to a dedicated filter
+    # function in visuals/filters as discussed in #1051
+    def _build_interpolation(self):
+        """Rebuild the _data_lookup_fn for different interpolations."""
+        interpolation = self._interpolation
+        self._data_lookup_fn = self._interpolation_fun[interpolation]
+        try:
+            self.shared_program.frag['get_data'] = self._data_lookup_fn
+        except Exception as e:
+            print(e)
+
+        # only 'linear' uses 'linear' texture interpolation
+        if interpolation == 'linear':
+            texture_interpolation = 'linear'
+        else:
+            # 'nearest' (and also 'linear') doesn't use spatial_filters.frag
+            # so u_kernel and shape setting is skipped
+            texture_interpolation = 'nearest'
+            if interpolation != 'nearest':
+                self.shared_program['u_kernel'] = self._kerneltex
+                self._data_lookup_fn['shape'] = self._last_data.shape[:3][::-1]
+
+        if self._texture.interpolation != texture_interpolation:
+            self._texture.interpolation = texture_interpolation
+
+        self._data_lookup_fn['texture'] = self._texture
+
+        self._need_interpolation_update = False
+
     @staticmethod
     @lru_cache(maxsize=10)
-    def _build_clipping_planes_func(n_planes):
+    def _build_clipping_planes_glsl(n_planes: int) -> str:
         """Build the code snippet used to clip the volume based on self.clipping_planes."""
         func_template = '''
             float clip_planes(vec3 loc, vec3 vol_shape) {{
-                vec3 loc_transf = $clip_transform(vec4(loc, 1)).xyz;
-                float distance_from_clip = 1.0;
+                vec3 loc_transf = $clip_transform(vec4(loc * vol_shape, 1)).xyz;
+                float distance_from_clip = 3.4e38; // max float
                 {clips};
                 return distance_from_clip;
             }}
         '''
         # the vertex is considered clipped if on the "negative" side of the plane
         clip_template = '''
-            vec3 relative_vec{idx} = loc_transf - ( $clipping_plane_pos{idx} / vol_shape );
-            float distance_from_clip{idx} = dot(relative_vec{idx}, ($clipping_plane_norm{idx} * vol_shape));
+            vec3 relative_vec{idx} = loc_transf - $clipping_plane_pos{idx};
+            float distance_from_clip{idx} = dot(relative_vec{idx}, $clipping_plane_norm{idx});
             distance_from_clip = min(distance_from_clip{idx}, distance_from_clip);
             '''
         all_clips = []
         for idx in range(n_planes):
             all_clips.append(clip_template.format(idx=idx))
         formatted_code = func_template.format(clips=''.join(all_clips))
-        return Function(formatted_code)
+        return formatted_code
 
     @property
-    def clipping_planes(self):
+    def clipping_planes(self) -> np.ndarray:
         """The set of planes used to clip the volume. Values on the negative side of the normal are discarded.
 
         Each plane is defined by a position and a normal vector (magnitude is irrelevant). Shape: (n_planes, 2, 3).
@@ -924,12 +1005,12 @@ class VolumeVisual(Visual):
         return self._clipping_planes
 
     @clipping_planes.setter
-    def clipping_planes(self, value):
+    def clipping_planes(self, value: Optional[np.ndarray]):
         if value is None:
             value = np.empty([0, 2, 3])
         self._clipping_planes = value
 
-        self._clip_func = self._build_clipping_planes_func(len(value))
+        self._clip_func = Function(self._build_clipping_planes_glsl(len(value)))
         self.shared_program.frag['clip_with_planes'] = self._clip_func
 
         self._clip_func['clip_transform'] = self._clip_transform
@@ -939,7 +1020,7 @@ class VolumeVisual(Visual):
         self.update()
 
     @property
-    def clipping_planes_coord_system(self):
+    def clipping_planes_coord_system(self) -> str:
         """
         Coordinate system used by the clipping planes (see visuals.transforms.transform_system.py)
         """
@@ -991,19 +1072,19 @@ class VolumeVisual(Visual):
                              (known_methods, method))
         self._method = method
 
-        # $sample needs to be unset and re-set, since it's present inside the snippets.
+        # $get_data needs to be unset and re-set, since it's present inside the snippets.
         #       Program should probably be able to do this automatically
-        self.shared_program.frag['sample'] = None
+        self.shared_program.frag['get_data'] = None
         self.shared_program.frag['raycasting_setup'] = self._raycasting_setup_snippet
         self.shared_program.frag['before_loop'] = self._before_loop_snippet
         self.shared_program.frag['in_loop'] = self._in_loop_snippet
         self.shared_program.frag['after_loop'] = self._after_loop_snippet
         self.shared_program.frag['sampler_type'] = self._texture.glsl_sampler_type
-        self.shared_program.frag['sample'] = self._texture.glsl_sample
         self.shared_program.frag['cmap'] = Function(self._cmap.glsl_map)
         self.shared_program['texture2D_LUT'] = self.cmap.texture_lut()
         self.shared_program['u_mip_cutoff'] = self._mip_cutoff
         self.shared_program['u_minip_cutoff'] = self._minip_cutoff
+        self._need_interpolation_update = True
         self.update()
 
     @property
@@ -1229,3 +1310,6 @@ class VolumeVisual(Visual):
     def _prepare_draw(self, view):
         if self._need_vertex_update:
             self._create_vertex_data()
+
+        if self._need_interpolation_update:
+            self._build_interpolation()
